from typing import List
from typing import TYPE_CHECKING

from aiogram import types

from aiogram_media_group.storages.base import BaseStorage

if TYPE_CHECKING:
    import aioredis

try:
    import ujson as json
except ImportError:
    import json


class RedisStorage(BaseStorage):
    def __init__(self, connection: "aioredis.RedisConnection", prefix: str, ttl: int):
        self._connection = connection
        self._prefix = prefix
        self._ttl = ttl

    def _get_media_group_handled_key(self, media_group_id: str) -> str:
        return f"{self._prefix}:{media_group_id}:handled"

    def _get_media_group_messages_key(self, media_group_id: str) -> str:
        return f"{self._prefix}:{media_group_id}:messages"

    async def set_media_group_as_handled(self, media_group_id: str) -> bool:
        return await self._connection.execute(
            "SET",
            self._get_media_group_handled_key(media_group_id),
            1,
            b"EX",
            self._ttl,
            b"NX",
        )

    async def append_message_to_media_group(
        self, media_group_id: str, message: types.Message
    ):
        length = await self._connection.execute(
            "LPUSH",
            self._get_media_group_messages_key(media_group_id),
            json.dumps(message.to_python()),
        )

        if length == 1:
            await self._connection.execute(
                "EXPIRE",
                self._get_media_group_messages_key(media_group_id),
                self._ttl,
            )

    async def get_media_group_messages(
        self, media_group_id: str
    ) -> List[types.Message]:
        raw_messages = await self._connection.execute(
            "LRANGE", self._get_media_group_messages_key(media_group_id), 0, 10
        )
        messages = [types.Message(**json.loads(m)) for m in raw_messages]
        messages.sort(key=lambda m: m.message_id)

        return messages

    async def delete_media_group(self, media_group_id: str):
        await self._connection.execute(
            "DEL",
            self._get_media_group_handled_key(media_group_id),
            self._get_media_group_messages_key(media_group_id),
        )



from aiogram import types
from aiogram.dispatcher.filters import Filter


class MediaGroupFilter(Filter):
    async def check(self, message: types.Message):
        return message.media_group_id is not None



import asyncio
from functools import wraps
from typing import TYPE_CHECKING, Callable, Optional

from aiogram import types, Dispatcher
from aiogram.contrib.fsm_storage.memory import MemoryStorage as AiogramMemoryStorage
from aiogram.contrib.fsm_storage.redis import (
    RedisStorage as AiogramRedisStorage,
    RedisStorage2 as AiogramRedis2Storage,
)
from aiogram.dispatcher import FSMContext

from aiogram_media_group.storages.base import BaseStorage
from aiogram_media_group.storages.memory import MemoryStorage
from aiogram_media_group.storages.redis import RedisStorage

if TYPE_CHECKING:
    import aioredis


async def _get_storage_from_state(state: FSMContext, prefix, ttl):
    storage_type = type(state.storage)
    if storage_type is AiogramMemoryStorage:
        return MemoryStorage(data=state.storage.data, prefix=prefix)
    elif storage_type is AiogramRedisStorage:
        connection: aioredis.RedisConnection = await state.storage.redis()
        return RedisStorage(connection=connection, prefix=prefix, ttl=ttl)
    elif storage_type is AiogramRedis2Storage:
        redis: aioredis.Redis = await state.storage.redis()
        return RedisStorage(connection=redis.connection, prefix=prefix, ttl=ttl)
    else:
        raise ValueError(f"{storage_type} is unsupported storage")


async def _on_media_group_received(
    media_group_id: str,
    storage: BaseStorage,
    callback,
    *args,
    **kwargs,
):
    messages = await storage.get_media_group_messages(media_group_id)
    await storage.delete_media_group(media_group_id)

    return await callback(messages, *args, **kwargs)


def media_group_handler(
    func: Optional[Callable] = None,
    receive_timeout: float = 1.0,
    storage_prefix: str = "media-group",
    loop=None,
    storage_driver: Optional[BaseStorage] = None,
):
    def decorator(handler):
        @wraps(handler)
        async def wrapper(message: types.Message, *args, **kwargs):
            if message.media_group_id is None:
                raise ValueError("Not a media group message")

            if loop is None:
                event_loop = asyncio.get_event_loop()
            else:
                event_loop = loop

            if storage_driver is not None:
                storage = storage_driver
            else:
                ttl = int(receive_timeout * 2)
                if ttl < 1:
                    ttl = 1

                state = Dispatcher.get_current().current_state()
                storage = await _get_storage_from_state(
                    state, prefix=storage_prefix, ttl=ttl
                )

            if await storage.set_media_group_as_handled(message.media_group_id):
                event_loop.call_later(
                    receive_timeout,
                    asyncio.create_task,
                    _on_media_group_received(
                        message.media_group_id, storage, handler, *args, **kwargs
                    ),
                )

            await storage.append_message_to_media_group(message.media_group_id, message)

        return wrapper

    if callable(func):
        return decorator(func)

    return decorator



from typing import List, Dict

from aiogram import types

from aiogram_media_group.storages.base import BaseStorage


class MemoryStorage(BaseStorage):
    def __init__(self, data: Dict, prefix: str):
        self._data = data
        self._prefix = prefix

    async def set_media_group_as_handled(self, media_group_id: str) -> bool:
        try:
            _ = self._data[self._prefix][media_group_id]
            return False
        except KeyError:
            self._data.setdefault(self._prefix, {}).setdefault(media_group_id, [])
            return True

    async def append_message_to_media_group(
        self, media_group_id: str, message: types.Message
    ):
        self._data[self._prefix][media_group_id].append(message)

    async def get_media_group_messages(
        self, media_group_id: str
    ) -> List[types.Message]:
        return self._data[self._prefix][media_group_id]

    async def delete_media_group(self, media_group_id: str):
        self._data[self._prefix].pop(media_group_id, None)



from abc import abstractmethod, ABC
from typing import List

from aiogram import types


class BaseStorage(ABC):
    @abstractmethod
    async def set_media_group_as_handled(self, media_group_id: str) -> bool:
        pass

    @abstractmethod
    async def append_message_to_media_group(
        self, media_group_id: str, message: types.Message
    ):
        pass

    @abstractmethod
    async def get_media_group_messages(
        self, media_group_id: str
    ) -> List[types.Message]:
        pass

    @abstractmethod
    async def delete_media_group(self, media_group_id: str):
        pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aiogram_media_group', 'aiogram_media_group.storages']

package_data = \
{'': ['*']}

install_requires = \
['aiogram>=2.11.2,<3.0.0']

setup_kwargs = {
    'name': 'aiogram-media-group',
    'version': '0.2.0',
    'description': 'AIOGram handler for media groups (also known as albums)',
    'long_description': '# aiogram-media-group\nAIOGram handler for media groups (also known as albums)\n\n### Features\n- Memory and Redis storage drivers supported\n- Ready to work with multiple bot instances\n\n### Install\n\n```bash\npip install aiogram-media-group\n# or\npoetry add aiogram-media-group\n```\n\n### Usage\n\nMinimal usage example:\n```python\nfrom aiogram_media_group import MediaGroupFilter, media_group_handler\n\n@dp.message_handler(MediaGroupFilter(), content_types=ContentType.PHOTO)\n@media_group_handler\nasync def album_handler(messages: List[types.Message]):\n    for message in messages:\n        print(message)\n```\n\nCheckout [echo-bot](./examples/echo.py) for complete usage example ',
    'author': 'deptyped',
    'author_email': 'deptyped@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/deptyped/aiogram-media-group',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
